#!/usr/bin/env python

# This is a script that creates a minimal jarfile from a python file.

import argparse
import os.path
import zipfile
import sys
import subprocess

# The base java directory.
JAVA = os.path.dirname(os.path.abspath(sys.argv[0]))

def zipinfo(fn, compression):
    """
    Create a canonical zipinfo object corresponding to filename.
    """

    rv = zipfile.ZipInfo(fn)
    rv.date_time = (2009, 9, 30, 5, 30, 0)
    rv.compress_type = compression
    
    return rv


def main():
    ap = argparse.ArgumentParser(description="Create a jar file from a java class, including all of the com.solartechnology dependencies used by that class.")
    ap.add_argument("jar", action="store", help="The name of the jarfile to use.")
    ap.add_argument("classname", action="store", help="The names of the main class to use.")
    ap.add_argument("files", action='store', default=[], nargs='*', help="Additional files to add to the jar.")
    ap.add_argument("--manifest", action="store", default=None, help="A set of lines to prepend to the manifest file.")
    ap.add_argument("--deflate", action="store_true", help="Compress the jar file.")

    args = ap.parse_args()

    # Extract apache river.
    if not os.path.exists("apache-river"):
        print "Extracting apache river."

        zf = zipfile.ZipFile("apache-river.zip", "r")
        zf.extractall(JAVA)
        zf.close()

        os.rename(
            os.path.join(JAVA, "apache-river-2.1.2-SNAPSHOT-incubating"),
            os.path.join(JAVA, "apache-river"))


    # Determine the files that the classes depend on.
    classfiles = [ ]

    classpath = ":".join([
        JAVA,
        os.path.join(JAVA, "../devroot/usr/share/classpath/glibj.zip"),
        os.path.join(JAVA, "gson-2.2.2.jar"),
        os.path.join(JAVA, "mongo-2.7.2.jar"),
        os.path.join(JAVA, "geronimo-javamail-1.4-provider.jar"),
        os.path.join(JAVA, "geronimo-javamail-1.4-spec.jar"),
        os.path.join(JAVA, "morphia-0.99.1-SNAPSHOT.jar"),
        os.path.join(JAVA, "swingx-core-1.6.2.jar"),
        os.path.join(JAVA, "swingx-ws-1.0.jar"),
        ])

    cmd = [
        "java",
        "-jar", os.path.join(JAVA, "apache-river/lib/classdep.jar"),
        "-cp", classpath,
        "-in", "com.solartechnology",
        "-hide", "com.google.code",
        "-hide", "javax.mail",
        "-hide", "com.mongodb",
        "-files", args.classname,
        ]

    p = subprocess.Popen(cmd, stdout=subprocess.PIPE)

    for l in p.stdout:
        l = l.strip()
        classfiles.append(l)

    p.wait()

    
    
    # Create the jar file.
    if args.deflate:
        compression = zipfile.ZIP_DEFLATED
    else:
        compression = zipfile.ZIP_STORED

    jar = zipfile.ZipFile(args.jar, "w", compression)


    # Write the manifest.
    manifest = [
        "Manifest-Version: 1.0",
        "Created-By: 1.6.0_22 (Solar Technology Inc.)",
        ]

    if args.manifest:
        f = file(args.manifest, "r")
        for l in f:
            l = l.strip()
            if not l:
                continue

            manifest.append(l)

    manifest = "\n".join(manifest) + "\n\n"
    jar.writestr(zipinfo("META-INF/MANIFEST.MF", compression), manifest)


    # Strip ./ from args.files.
    files = [ ]
    for i in args.files:
        if ".svn" in i:
            continue

        if i.startswith("./"):
            i = i[2:]
        files.append(i)



    # Write the classfiles.
    for fn in classfiles + files:

        f = file(os.path.join(JAVA, fn), "r")
        data = f.read()
        f.close()

        jar.writestr(zipinfo(fn, compression), data)

    # Done.
    jar.close()

    print "Added", len(classfiles), " classes and ", len(args.files), "files to", args.jar

if __name__ == "__main__":
    main()
