#!/usr/bin/env python

import serial
import threading
import sys
import os
import time

# The serial port we have open.
sport = None

# A condition that we notify in order to quit cleanly.
quit_condition = threading.Condition()

def quit():
    """
    Force the program to quit, by notifying quit_condition.
    """

    quit_condition.acquire()
    quit_condition.notify()
    quit_condition.release()
    
def stdin_to_serial():
    """
    This thread is responsible for sending data from standard input to the
    serial port. It reads a character, sends it, and repeats forever. It
    forces a quit if stdin has closed.
    """

    while True:
        c = sys.stdin.read(1)

        if not c:
            quit()
            return
            
        sport.write(c)


def serial_to_stdout():
    """
    This thread is responsible for reading from the serial port, and
    sending data to standard output. It forces a quit when CD goes from
    high to low. (Basically, when the modem hangs up.)
    """

    old_cd = False
    
    while True:

        # When CD drops, exit.
        cd = sport.getCD()

        if old_cd and not cd:
            quit()
            return
            
        old_cd = cd

        # Now, try to read a byte and write it out.
        c = sport.read(1)

        if not c:
            quit()
            return
        
        if c:
            sys.stdout.write(c)
            sys.stdout.flush()
            

def main():
    
    if len(sys.argv) != 3:
        print "usage: %s <port> <bitrate>" % sys.argv[0]
        sys.exit(-1)

    # Switch stdin and stdout to binary files.
    try:
        import msvcrt
        msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)
        msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
    except ImportError:
        pass
        
    global sport

    # Open the serial port. Will write an exception to stderr if we can't
    # open it.
    sport = serial.Serial(
        port=sys.argv[1],
        baudrate=int(sys.argv[2]),
        timeout=1,
        # rtscts=True
        )

    # Raise DTR.
    sport.setDTR(True)

    # Clear out the sport.
    while True:
        c = sport.read(1)
        if not c:
            break
        
    sport.setTimeout(None)

    # Threads.
    t1 = threading.Thread(target=stdin_to_serial)
    t1.setDaemon(True)
    t1.start()

    t2 = threading.Thread(target=serial_to_stdout)
    t2.setDaemon(True)
    t2.start()

    # Wait for a thread to notify us. Since the threads are daemon, the
    # main thread quitting shuts down the whole program.
    quit_condition.acquire()
    quit_condition.wait()
    quit_condition.release()

    # Be sure we hang up the modem.
    sport.setDTR(False)
    time.sleep(1)
    
    os._exit(0)
    
if __name__ == "__main__":
    main()
    
